@@ -43,7 +43,6 @@ typedef struct
 #define ENABLE_SCRIPTS_SETTING "enable-javascript"
 #define DEFAULT_ENCODING_SETTING "default-charset"
 static WebKitSettings *webkit_settings = NULL;
-static WebKitWebViewGroup *web_view_group = NULL;
 
 static void
 user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
@@ -54,9 +53,14 @@ user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
 
   bytes = g_output_stream_splice_finish (output_stream, result, NULL);
   if (bytes > 0) {
-    webkit_web_view_group_add_user_style_sheet (web_view_group,
-                                                g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
-                                                NULL, NULL, NULL, WEBKIT_INJECTED_CONTENT_FRAMES_ALL);
+    WebKitUserStyleSheet *style_sheet;
+
+    style_sheet = webkit_user_style_sheet_new (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
+                                               WEBKIT_USER_CONTENT_INJECT_ALL_FRAMES, WEBKIT_USER_STYLE_LEVEL_USER,
+                                               NULL, NULL);
+    webkit_user_content_manager_add_style_sheet (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())),
+                                                 style_sheet);
+    webkit_user_style_sheet_unref (style_sheet);
   }
 }
 
@@ -94,7 +98,7 @@ webkit_pref_callback_user_stylesheet (GSettings *settings,
   value = g_settings_get_boolean (settings, key);
 
   if (!value)
-    webkit_web_view_group_remove_all_user_style_sheets (web_view_group);
+    webkit_user_content_manager_remove_all_style_sheets (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())));
   else {
     GFile *file;
     char *filename;
@@ -199,10 +203,6 @@ webkit_pref_callback_user_agent (GSettings *settings,
                                                              "Epiphany", VERSION);
 }
 
-/* This doesn't contain WebKit2 specific API, but it's only used inside
- * HAVE_WEBKIT2 blocks, so it gives a compile warning when building
- * with WebKit1.
- */
 static gdouble
 get_screen_dpi (GdkScreen *screen)
 {
@@ -245,7 +245,7 @@ webkit_pref_callback_font_size (GSettings *settings,
   int size = 9; /* FIXME: What to use here? */
 
   char *schema = NULL;
-  g_object_get (settings, "schema", &schema, NULL);
+  g_object_get (settings, "schema-id", &schema, NULL);
 
   /* If we are changing a GNOME font value and we are not using GNOME fonts in
    * Epiphany, return. */
@@ -282,7 +282,7 @@ webkit_pref_callback_font_family (GSettings *settings,
   char *value = NULL;
 
   char *schema = NULL;
-  g_object_get (settings, "schema", &schema, NULL);
+  g_object_get (settings, "schema-id", &schema, NULL);
 
   /* If we are changing a GNOME font value and we are not using GNOME fonts in
    * Epiphany, return. */
@@ -528,19 +528,17 @@ static const PrefData webkit_pref_entries[] =
       webkit_pref_callback_cookie_accept_policy },
   };
 
-void
-ephy_embed_prefs_init (void)
+static gpointer
+ephy_embed_prefs_init (gpointer user_data)
 {
   int i;
-  web_view_group = webkit_web_view_group_new ("Ephy WebView Group");
-  webkit_settings = webkit_web_view_group_get_settings (web_view_group);
 
-  g_object_set (webkit_settings,
-                "enable-developer-extras", TRUE,
-                "enable-fullscreen", TRUE,
-                "enable-site-specific-quirks", TRUE,
-                "enable-dns-prefetching", TRUE,
-                NULL);
+  webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
+                                                       "enable-fullscreen", TRUE,
+                                                       "enable-site-specific-quirks", TRUE,
+                                                       "enable-dns-prefetching", TRUE,
+                                                       "javascript-can-open-windows-automatically", TRUE,
+                                                       NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
     GSettings *settings;
@@ -567,10 +565,6 @@ ephy_embed_prefs_init (void)
                    EPHY_PREFS_ENABLE_CARET_BROWSING,
                    webkit_settings, "enable-caret-browsing",
                    G_SETTINGS_BIND_GET);
-  g_settings_bind (EPHY_SETTINGS_WEB,
-                   EPHY_PREFS_WEB_ENABLE_POPUPS,
-                   webkit_settings, "javascript-can-open-windows-automatically",
-                   G_SETTINGS_BIND_GET);
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_PLUGINS,
                    webkit_settings, "enable-plugins",
@@ -595,16 +589,13 @@ ephy_embed_prefs_init (void)
                    EPHY_PREFS_ENABLE_SMOOTH_SCROLLING,
                    webkit_settings, "enable-smooth-scrolling",
                    G_SETTINGS_BIND_GET);
+  return webkit_settings;
 }
 
-void
-ephy_embed_prefs_shutdown (void)
+WebKitSettings *
+ephy_embed_prefs_get_settings (void)
 {
-  g_object_unref (web_view_group);
-}
+  static GOnce once_init = G_ONCE_INIT;
 
-WebKitWebViewGroup *
-ephy_embed_prefs_get_web_view_group (void)
-{
-  return web_view_group;
+  return g_once (&once_init, ephy_embed_prefs_init, NULL);
 }
