@@ -27,11 +27,13 @@
 #include "ephy-embed.h"
 
 #include "ephy-debug.h"
-#include "ephy-download.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
+#include "ephy-embed-utils.h"
+#include "ephy-find-toolbar.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
+#include "ephy-string.h"
 #include "ephy-web-view.h"
 #include "nautilus-floating-bar.h"
 
@@ -45,6 +47,8 @@ static void     ephy_embed_restored_window_cb  (EphyEmbedShell *shell,
 #define EPHY_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED, EphyEmbedPrivate))
 
 #define EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION "tab_message"
+#define MAX_TITLE_LENGTH 512 /* characters */
+#define EMPTY_PAGE_TITLE _("Blank page") /* Title for the empty page */
 
 typedef struct {
   gchar *text;
@@ -54,6 +58,7 @@ typedef struct {
 
 struct _EphyEmbedPrivate
 {
+  EphyFindToolbar *find_toolbar;
   GtkBox *top_widgets_vbox;
   GtkPaned *paned;
   WebKitWebView *web_view;
@@ -63,10 +68,9 @@ struct _EphyEmbedPrivate
   GtkWidget *fullscreen_message_label;
   char *fullscreen_string;
 
+  char *title;
   WebKitURIRequest *delayed_request;
 
-  GtkWidget *overview;
-  guint overview_mode : 1;
   GSList *messages;
   GSList *keys;
 
@@ -87,7 +91,8 @@ struct _EphyEmbedPrivate
 enum
 {
   PROP_0,
-  PROP_OVERVIEW_MODE,
+  PROP_WEB_VIEW,
+  PROP_TITLE,
 };
 
 G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
@@ -151,7 +156,7 @@ ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
 {
   EphyEmbedPrivate *priv = embed->priv;
 
-  nautilus_floating_bar_set_label (NAUTILUS_FLOATING_BAR (priv->floating_bar), label);
+  nautilus_floating_bar_set_primary_label (NAUTILUS_FLOATING_BAR (priv->floating_bar), label);
 
   if (label == NULL || label[0] == '\0') {
     gtk_widget_hide (priv->floating_bar);
@@ -241,22 +246,58 @@ remove_from_destroy_list_cb (GtkWidget *widget, EphyEmbed *embed)
   embed->priv->destroy_on_transition_list = list;
 }
 
+static void
+ephy_embed_set_title (EphyEmbed *embed,
+                      const char *title)
+{
+  EphyEmbedPrivate *priv = embed->priv;
+  char *new_title;
+
+  new_title = g_strdup (title);
+  if (new_title == NULL || g_strstrip (new_title)[0] == '\0') {
+    const char *address;
+
+    g_free (new_title);
+    new_title = NULL;
+
+    address = ephy_web_view_get_address (EPHY_WEB_VIEW (priv->web_view));
+    if (address && strcmp (address, "about:blank") != 0)
+      new_title = ephy_embed_utils_get_title_from_address (address);
+
+    if (new_title == NULL || new_title[0] == '\0') {
+      g_free (new_title);
+      new_title = g_strdup (EMPTY_PAGE_TITLE);
+    }
+  }
+
+  g_free (priv->title);
+  priv->title = ephy_string_shorten (new_title, MAX_TITLE_LENGTH);
+
+  g_object_notify (G_OBJECT (embed), "title");
+}
+
+static void
+web_view_title_changed_cb (WebKitWebView *web_view,
+                           GParamSpec *spec,
+                           EphyEmbed *embed)
+{
+  ephy_embed_set_title (embed, webkit_web_view_get_title (web_view));
+}
+
 static void
 load_changed_cb (WebKitWebView *web_view,
                  WebKitLoadEvent load_event,
                  EphyEmbed *embed)
 {
   switch (load_event) {
-  case WEBKIT_LOAD_STARTED: {
-    const char *uri;
-
-    uri = webkit_web_view_get_uri (web_view);
-    ephy_embed_set_overview_mode (embed, g_strcmp0 (uri, "ephy-about:overview") == 0);
-    break;
-  }
   case WEBKIT_LOAD_COMMITTED:
     ephy_embed_destroy_top_widgets (embed);
     break;
+  case WEBKIT_LOAD_FINISHED:
+    if (ephy_web_view_get_is_blank (EPHY_WEB_VIEW (web_view)) ||
+        !webkit_web_view_get_title (web_view))
+      ephy_embed_set_title (embed, NULL);
+    break;
   default:
     break;
   }
@@ -289,14 +330,17 @@ fullscreen_message_label_hide (EphyEmbed *embed)
 void
 ephy_embed_entering_fullscreen (EphyEmbed *embed)
 {
-  gtk_widget_show (embed->priv->fullscreen_message_label);
+  if (!g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN, EPHY_PREFS_LOCKDOWN_FULLSCREEN)) {
+    gtk_widget_show (embed->priv->fullscreen_message_label);
 
-  if (embed->priv->fullscreen_message_id)
-    g_source_remove (embed->priv->fullscreen_message_id);
+    if (embed->priv->fullscreen_message_id)
+      g_source_remove (embed->priv->fullscreen_message_id);
 
-  embed->priv->fullscreen_message_id = g_timeout_add_seconds (5,
-                                                              (GSourceFunc)fullscreen_message_label_hide,
-                                                              embed);
+    embed->priv->fullscreen_message_id = g_timeout_add_seconds (5,
+                                                                (GSourceFunc)fullscreen_message_label_hide,
+                                                                embed);
+    g_source_set_name_by_id (embed->priv->fullscreen_message_id, "[epiphany] fullscreen_message_label_hide");
+  }
 }
 
 void
@@ -378,6 +422,7 @@ ephy_embed_finalize (GObject *object)
   priv->keys = NULL;
 
   g_free (embed->priv->fullscreen_string);
+  g_free (priv->title);
 
   G_OBJECT_CLASS (ephy_embed_parent_class)->finalize (object);
 }
@@ -392,8 +437,11 @@ ephy_embed_set_property (GObject *object,
 
   switch (prop_id)
   {
-  case PROP_OVERVIEW_MODE:
-    ephy_embed_set_overview_mode (embed, g_value_get_boolean (value));
+  case PROP_WEB_VIEW:
+    embed->priv->web_view = g_value_get_object (value);
+    break;
+  case PROP_TITLE:
+    ephy_embed_set_title (embed, g_value_dup_string (value));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -411,8 +459,11 @@ ephy_embed_get_property (GObject *object,
 
   switch (prop_id)
   {
-  case PROP_OVERVIEW_MODE:
-    g_value_set_boolean (value, ephy_embed_get_overview_mode (embed));
+  case PROP_WEB_VIEW:
+    g_value_set_object (value, ephy_embed_get_web_view (embed));
+    break;
+  case PROP_TITLE:
+    g_value_set_string (value, ephy_embed_get_title (embed));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -420,6 +471,17 @@ ephy_embed_get_property (GObject *object,
   }
 }
 
+static void
+ephy_embed_find_toolbar_close_cb (EphyFindToolbar *toolbar,
+                                  EphyEmbed *embed)
+{
+  EphyEmbedPrivate *priv = embed->priv;
+
+  ephy_find_toolbar_close (priv->find_toolbar);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+}
+
 static void
 ephy_embed_class_init (EphyEmbedClass *klass)
 {
@@ -433,18 +495,21 @@ ephy_embed_class_init (EphyEmbedClass *klass)
   object_class->get_property = ephy_embed_get_property;
   widget_class->grab_focus = ephy_embed_grab_focus;
 
-/**
- * EphyEmbed:overview-mode:
- *
- * If %TRUE activates the overview mode in this #EphyEmbed.
- **/
   g_object_class_install_property (object_class,
-                                   PROP_OVERVIEW_MODE,
-                                   g_param_spec_boolean ("overview-mode",
-                                                         "Overview mode",
-                                                         "Whether the embed is showing the overview",
-                                                         FALSE,
-                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                   PROP_WEB_VIEW,
+                                   g_param_spec_object ("web-view",
+                                                        "Web View",
+                                                        "The WebView contained in the embed",
+                                                        EPHY_TYPE_WEB_VIEW,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property (object_class,
+                                   PROP_TITLE,
+                                   g_param_spec_string ("title",
+                                                        "Title",
+                                                        "The embed's title",
+                                                        NULL,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
   g_type_class_add_private (G_OBJECT_CLASS (klass), sizeof(EphyEmbedPrivate));
 }
@@ -467,16 +532,6 @@ ephy_embed_attach_inspector_cb (WebKitWebInspector *inspector,
   return TRUE;
 }
 
-void
-ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
-{
-  EphyDownload *download;
-
-  download = ephy_download_new_for_uri (url, NULL);
-  ephy_download_set_auto_destination (download);
-  ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_OPEN);
-}
-
 static void
 ephy_embed_set_fullscreen_message (EphyEmbed *embed,
                                    gboolean is_html5_fullscreen)
@@ -543,32 +598,11 @@ status_message_notify_cb (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed
       over a series of links, the overlay widget doesn't flicker on and off. */
     if (priv->pop_statusbar_later_source_id == 0) {
       priv->pop_statusbar_later_source_id = g_timeout_add (250, pop_statusbar_later_cb, embed);
+      g_source_set_name_by_id (priv->pop_statusbar_later_source_id, "[epiphany] pop_statusbar_later_cb");
     }
   }
 }
 
-static void
-window_geometry_changed (WebKitWindowProperties *properties, GParamSpec *pspec, EphyEmbed *embed)
-{
-  GtkWidget *window;
-  gboolean is_popup;
-  GdkRectangle geometry;
-
-  window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
-  if (!window || !gtk_widget_is_toplevel (window))
-    return;
-
-  g_object_get (window, "is-popup", &is_popup, NULL);
-  if (!is_popup)
-    return;
-
-  webkit_window_properties_get_geometry (properties, &geometry);
-  if (geometry.x >= 0 && geometry.y >= 0)
-    gtk_window_move (GTK_WINDOW (window), geometry.x, geometry.y);
-  if (geometry.width > 0 && geometry.height > 0)
-    gtk_window_resize (GTK_WINDOW (window), geometry.width, geometry.height);
-}
-
 static gboolean
 clear_progress_cb (EphyEmbed *embed)
 {
@@ -594,17 +628,20 @@ progress_update (EphyWebView *view, GParamSpec *pspec, EphyEmbed *embed)
 
   uri = webkit_web_view_get_uri (priv->web_view);
   if (!uri || g_str_has_prefix (uri, "ephy-about:") ||
-      g_str_has_prefix (uri, "about:"))
+      g_str_has_prefix (uri, "about:")) {
+    gtk_widget_hide (priv->progress);
     return;
+  }
 
   progress = webkit_web_view_get_estimated_load_progress (priv->web_view);
   loading = ephy_web_view_is_loading (EPHY_WEB_VIEW (priv->web_view));
 
-  if (progress == 1.0 || !loading)
+  if (progress == 1.0 || !loading) {
     priv->clear_progress_source_id = g_timeout_add (500,
                                                     (GSourceFunc)clear_progress_cb,
                                                     embed);
-  else
+    g_source_set_name_by_id (priv->clear_progress_source_id, "[epiphany] clear_progress_cb");
+  } else
     gtk_widget_show (priv->progress);
 
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (priv->progress),
@@ -653,8 +690,6 @@ ephy_embed_constructed (GObject *object)
   EphyEmbedPrivate *priv = embed->priv;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   GtkWidget *paned;
-  WebKitWebView *web_view;
-  WebKitWindowProperties *window_properties;
   WebKitWebInspector *inspector;
   GtkWidget *overlay;
 
@@ -665,26 +700,12 @@ ephy_embed_constructed (GObject *object)
                     G_CALLBACK (ephy_embed_mapped_cb), NULL);
 
   /* Skeleton */
-  web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
   overlay = gtk_overlay_new ();
 
   gtk_widget_add_events (overlay, 
                          GDK_ENTER_NOTIFY_MASK |
                          GDK_LEAVE_NOTIFY_MASK);
-  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (web_view));
-
-  /* The overview. In incognito mode we don't use it. */
-  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) !=
-      EPHY_EMBED_SHELL_MODE_INCOGNITO) {
-    priv->overview = ephy_overview_new ();
-    gtk_widget_set_halign (priv->overview, GTK_ALIGN_FILL);
-    gtk_widget_set_valign (priv->overview, GTK_ALIGN_FILL);
-    gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->overview);
-
-    g_object_bind_property (embed, "overview-mode",
-                            priv->overview, "visible",
-                            G_BINDING_SYNC_CREATE);
-  }
+  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (priv->web_view));
 
   /* Floating message popup for fullscreen mode. */
   priv->fullscreen_message_label = gtk_label_new (NULL);
@@ -696,7 +717,7 @@ ephy_embed_constructed (GObject *object)
   ephy_embed_set_fullscreen_message (embed, FALSE);
 
   /* statusbar is hidden by default */
-  priv->floating_bar = nautilus_floating_bar_new (NULL, FALSE);
+  priv->floating_bar = nautilus_floating_bar_new (NULL, NULL, FALSE);
   gtk_widget_set_halign (priv->floating_bar, GTK_ALIGN_START);
   gtk_widget_set_valign (priv->floating_bar, GTK_ALIGN_END);
   gtk_widget_set_no_show_all (priv->floating_bar, TRUE);
@@ -710,10 +731,18 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_set_valign (priv->progress, GTK_ALIGN_START);
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->progress);
 
+  priv->find_toolbar = ephy_find_toolbar_new (priv->web_view);
+  g_signal_connect (priv->find_toolbar, "close",
+                    G_CALLBACK (ephy_embed_find_toolbar_close_cb),
+                    embed);
+
+  gtk_box_pack_start (GTK_BOX (embed),
+                      GTK_WIDGET (priv->find_toolbar),
+                      FALSE, FALSE, 0);
+
   paned = GTK_WIDGET (priv->paned);
 
-  priv->web_view = web_view;
-  priv->progress_update_handler_id = g_signal_connect (web_view, "notify::estimated-load-progress",
+  priv->progress_update_handler_id = g_signal_connect (priv->web_view, "notify::estimated-load-progress",
                                                        G_CALLBACK (progress_update), object);
   gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (overlay),
                    TRUE, FALSE);
@@ -724,26 +753,22 @@ ephy_embed_constructed (GObject *object)
   gtk_box_pack_start (GTK_BOX (embed), paned, TRUE, TRUE, 0);
 
   gtk_widget_show (GTK_WIDGET (priv->top_widgets_vbox));
-  gtk_widget_show (GTK_WIDGET (web_view));
+  gtk_widget_show (GTK_WIDGET (priv->web_view));
   gtk_widget_show_all (paned);
 
-  g_object_connect (web_view,
+  g_object_connect (priv->web_view,
+                    "signal::notify::title", G_CALLBACK (web_view_title_changed_cb), embed,
                     "signal::load-changed", G_CALLBACK (load_changed_cb), embed,
                     "signal::enter-fullscreen", G_CALLBACK (entering_fullscreen_cb), embed,
                     "signal::leave-fullscreen", G_CALLBACK (leaving_fullscreen_cb), embed,
                     NULL);
 
-  priv->status_handler_id = g_signal_connect (web_view, "notify::status-message",
+  priv->status_handler_id = g_signal_connect (priv->web_view, "notify::status-message",
                                               G_CALLBACK (status_message_notify_cb),
                                               embed);
-  /* Window properties */
-  window_properties = webkit_web_view_get_window_properties (web_view);
-  g_signal_connect (window_properties, "notify::geometry",
-                    G_CALLBACK (window_geometry_changed),
-                    embed);
 
   /* The inspector */
-  inspector = webkit_web_view_get_inspector (web_view);
+  inspector = webkit_web_view_get_inspector (priv->web_view);
 
   g_signal_connect (inspector, "attach",
                     G_CALLBACK (ephy_embed_attach_inspector_cb),
@@ -783,6 +808,23 @@ ephy_embed_get_web_view (EphyEmbed *embed)
   return EPHY_WEB_VIEW (embed->priv->web_view);
 }
 
+/**
+ * ephy_embed_get_find_toolbar:
+ * @embed: and #EphyEmbed
+ * 
+ * Returns the #EphyFindToolbar wrapped by @embed.
+ * 
+ * Returns: (transfer none): an #EphyFindToolbar
+ **/
+EphyFindToolbar*
+ephy_embed_get_find_toolbar (EphyEmbed *embed)
+{
+  g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+
+  return EPHY_FIND_TOOLBAR (embed->priv->find_toolbar);
+}
+
+
 /**
  * ephy_embed_add_top_widget:
  * @embed: an #EphyEmbed
@@ -835,31 +877,6 @@ ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
                         GTK_WIDGET (widget));
 }
 
-void
-ephy_embed_set_overview_mode (EphyEmbed *embed, gboolean overview_mode)
-{
-  EphyEmbedPrivate *priv;
-
-  g_return_if_fail (EPHY_IS_EMBED (embed));
-
-  priv = embed->priv;
-
-  if (priv->overview_mode == overview_mode)
-    return;
-
-  priv->overview_mode = overview_mode;
-
-  g_object_notify (G_OBJECT (embed), "overview-mode");
-}
-
-gboolean
-ephy_embed_get_overview_mode (EphyEmbed *embed)
-{
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
-
-  return embed->priv->overview_mode;
-}
-
 /**
  * ephy_embed_set_delayed_load_request:
  * @embed: a #EphyEmbed
@@ -896,18 +913,10 @@ ephy_embed_has_load_pending (EphyEmbed *embed)
   return !!embed->priv->delayed_request;
 }
 
-/**
- * ephy_embed_get_overview:
- * @embed: a #EphyEmbed
- *
- * Gets the #EphyOverview in this @embed
- *
- * Returns: (transfer none): the overview widget
- **/
-EphyOverview *
-ephy_embed_get_overview (EphyEmbed *embed)
+const char *
+ephy_embed_get_title (EphyEmbed *embed)
 {
   g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
 
-  return EPHY_OVERVIEW (embed->priv->overview);
+  return embed->priv->title;
 }
