@@ -24,10 +24,12 @@
 #include "config.h"
 #include "ephy-location-entry.h"
 
+#include "ephy-widgets-type-builtins.h"
 #include "ephy-about-handler.h"
 #include "ephy-debug.h"
 #include "ephy-dnd.h"
 #include "ephy-gui.h"
+#include "ephy-lib-type-builtins.h"
 #include "ephy-signal-accumulator.h"
 
 #include <libsoup/soup.h>
@@ -35,11 +37,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
-#ifdef HAVE_WEBKIT2
 #include <webkit2/webkit2.h>
-#else
-#include <webkit/webkit.h>
-#endif
 
 /**
  * SECTION:ephy-location-entry
@@ -53,7 +51,6 @@
 
 struct _EphyLocationEntryPrivate
 {
-	GIcon *lock_gicon;
 	GdkPixbuf *favicon;
 	GtkTreeModel *model;
 	GtkEntryCompletion *completion;
@@ -81,8 +78,10 @@ struct _EphyLocationEntryPrivate
 	guint original_address : 1;
 	guint apply_colors : 1;
 	guint needs_reset : 1;
-	guint show_lock : 1;
 	guint show_favicon : 1;
+
+	GtkTargetList *drag_targets;
+	GdkDragAction drag_actions;
 };
 
 static const GtkTargetEntry url_drag_types [] =
@@ -110,8 +109,7 @@ enum
 	PROP_0,
 	PROP_LOCATION,
 	PROP_FAVICON,
-	PROP_LOCK_STOCK,
-	PROP_SHOW_LOCK,
+	PROP_SECURITY_LEVEL,
 	PROP_SHOW_FAVICON
 };
 
@@ -145,13 +143,9 @@ ephy_location_entry_set_property (GObject *object,
 		ephy_location_entry_set_favicon (entry,
 						 g_value_get_object (value));
 		break;
-	case PROP_LOCK_STOCK:
-		ephy_location_entry_set_lock_stock (entry,
-						    g_value_get_string (value));
-		break;
-	case PROP_SHOW_LOCK:
-		ephy_location_entry_set_show_lock (entry,
-						   g_value_get_boolean (value));
+	case PROP_SECURITY_LEVEL:
+		ephy_location_entry_set_security_level (entry,
+							g_value_get_enum (value));
 		break;
 	case PROP_SHOW_FAVICON:
 		ephy_location_entry_set_show_favicon (entry,
@@ -188,15 +182,15 @@ ephy_location_entry_finalize (GObject *object)
 	
 	g_free (priv->saved_text);
 
+	if (priv->drag_targets != NULL)
+	{
+		gtk_target_list_unref (priv->drag_targets);
+	}
+
 	if (priv->favicon != NULL)
 	{
 		g_object_unref (priv->favicon);
 	}
-	
-	if (priv->lock_gicon)
-	{
-		g_object_unref (priv->lock_gicon);
-	}
 
 	G_OBJECT_CLASS (ephy_location_entry_parent_class)->finalize (object);
 }
@@ -210,7 +204,7 @@ ephy_location_entry_get_preferred_width (GtkWidget       *widget,
 		*minimum_width = -1;
 
 	if (natural_width)
-		*natural_width = 518;
+		*natural_width = 848;
 }
 
 static void
@@ -251,30 +245,18 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	/**
-	* EphyLocationEntry:lock-stock-id:
+	* EphyLocationEntry:security-level:
 	*
-	* Stock id of the security icon.
+	* State of the security icon.
 	*/
 	g_object_class_install_property (object_class,
-					 PROP_LOCK_STOCK,
-					 g_param_spec_string  ("lock-stock-id",
-							       "Lock Stock ID",
-							       "Stock id of the security icon",
-							       NULL,
-							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
-
-	/**
-	* EphyLocationEntry:show-lock:
-	*
-	* If we should show the security icon.
-	*/
-	g_object_class_install_property (object_class,
-					 PROP_SHOW_LOCK,
-					 g_param_spec_boolean ("show-lock",
-							       "Show Lock",
-							       "If we should show the security icon",
-							       FALSE,
-							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+					 PROP_SECURITY_LEVEL,
+					 g_param_spec_enum  ("security-level",
+							     "Security level",
+							     "State of the security icon",
+							     EPHY_TYPE_SECURITY_LEVEL,
+							     EPHY_SECURITY_LEVEL_NO_SECURITY,
+							     G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_FAVICON,
@@ -390,13 +372,20 @@ update_favicon (EphyLocationEntry *lentry)
 	}
 	else if (priv->show_favicon)
 	{
+		const char *icon_name;
+
 		/* Here we could consider using fallback favicon that matches
 		 * the page MIME type, though text/html should be good enough
 		 * most of the time. See #337140
 		 */
+		if (gtk_entry_get_text_length (entry) > 0)
+			icon_name = "text-x-generic-symbolic";
+		else
+			icon_name = "edit-find-symbolic";
+
 		gtk_entry_set_icon_from_icon_name (entry,
 						   GTK_ENTRY_ICON_PRIMARY,
-						   "text-html");
+						   icon_name);
 	}
 	else
 	{
@@ -428,16 +417,25 @@ editable_changed_cb (GtkEditable *editable,
 static gboolean
 entry_key_press_cb (GtkEntry *entry,
 		    GdkEventKey *event,
-		    EphyLocationEntry *lentry)
+		    EphyLocationEntry *location_entry)
 {
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
 
+
 	if (event->keyval == GDK_KEY_Escape && state == 0)
 	{
-		ephy_location_entry_reset_internal (lentry, TRUE);
+		ephy_location_entry_reset_internal (location_entry, TRUE);
 		/* don't return TRUE since we want to cancel the autocompletion popup too */
 	}
 
+	if (event->keyval == GDK_KEY_l && state == GDK_CONTROL_MASK)
+	{
+		/* Make sure the location is activated on CTRL+l even when the
+		 * completion popup is shown and have an active keyboard grab.
+		 */
+		ephy_location_entry_activate (location_entry);
+	}
+
 	return FALSE;
 }
 
@@ -658,8 +656,21 @@ each_url_get_data_binder (EphyDragEachSelectedItemDataGet iteratee,
 	g_free (title);
 }
 
-#define DRAG_ICON_LAYOUT_PADDING	2
-#define DRAG_ICON_ICON_PADDING		4
+static void
+sanitize_location (char **url)
+{
+	char *str;
+
+	/* Do not show internal ephy-about: protocol to users */
+	if (g_str_has_prefix (*url, EPHY_ABOUT_SCHEME)) {
+		str = g_strdup_printf ("about:%s", *url + strlen (EPHY_ABOUT_SCHEME) + 1);
+		g_free (*url);
+		*url = str;
+	}
+}
+
+#define DRAG_ICON_LAYOUT_PADDING	5
+#define DRAG_ICON_ICON_PADDING		10
 #define DRAG_ICON_MAX_WIDTH_CHARS	32
 
 static cairo_surface_t *
@@ -682,10 +693,12 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	cairo_t *cr;
 	GtkStateFlags state;
 	GdkRGBA color;
+	GdkPixbuf *favicon;
 
 	state = gtk_widget_get_state_flags (widget);
 
 	g_signal_emit (entry, signals[GET_LOCATION], 0, &address);
+	sanitize_location (&address);
 	g_signal_emit (entry, signals[GET_TITLE], 0, &title);
 	if (address == NULL || title == NULL) return NULL;
 
@@ -705,9 +718,16 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	}
 
 	if (priv->favicon != NULL)
+		favicon = g_object_ref (priv->favicon);
+	else
+		favicon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+						    "text-x-generic-symbolic",
+						    16,
+						    0, NULL);
+	if (favicon != NULL)
 	{
-		icon_width = gdk_pixbuf_get_width (priv->favicon);
-		icon_height = gdk_pixbuf_get_height (priv->favicon);
+		icon_width = gdk_pixbuf_get_width (favicon);
+		icon_height = gdk_pixbuf_get_height (favicon);
 	}
 
 	context = gtk_widget_get_pango_context (widget);
@@ -729,13 +749,13 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 
 	pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
 
-	if (priv->favicon != NULL)
+	if (favicon != NULL)
 	{
 		favicon_offset_x = icon_width + (2 * DRAG_ICON_ICON_PADDING);
 	}
 
 	surface_width = layout_width + favicon_offset_x +
-			(DRAG_ICON_LAYOUT_PADDING * 2);
+			(DRAG_ICON_LAYOUT_PADDING * 3);
 	surface_height = MAX (layout_height, icon_height) +
 			(DRAG_ICON_LAYOUT_PADDING * 2);
 
@@ -755,14 +775,14 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	gdk_cairo_set_source_rgba (cr, &color);
 	cairo_fill (cr);
 
-	if (priv->favicon != NULL)
+	if (favicon != NULL)
 	{
 		double x;
 		double y;
 
 		x = 1 + DRAG_ICON_LAYOUT_PADDING + DRAG_ICON_ICON_PADDING;
-		y = 1 + DRAG_ICON_LAYOUT_PADDING + (surface_height - icon_height) / 2;
-		gdk_cairo_set_source_pixbuf (cr, priv->favicon, x, y);
+		y = (surface_height - icon_height) / 2;
+		gdk_cairo_set_source_pixbuf (cr, favicon, x, y);
 		cairo_rectangle (cr, x, y, icon_width, icon_height);
 		cairo_fill (cr);
 	}
@@ -780,6 +800,7 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	g_free (address);
 	g_free (title);
 	g_string_free (text, TRUE);
+	g_clear_object (&favicon);
 
 	return surface;
 }
@@ -869,29 +890,29 @@ static void
 ephy_location_entry_construct_contents (EphyLocationEntry *lentry)
 {
 	GtkWidget *entry = GTK_WIDGET (lentry);
-	GtkTargetList *targetlist;
+	EphyLocationEntryPrivate *priv = lentry->priv;
 
 	LOG ("EphyLocationEntry constructing contents %p", lentry);
 
 	/* Favicon */
-	targetlist = gtk_target_list_new (url_drag_types,
-					  G_N_ELEMENTS (url_drag_types));
+	priv->drag_targets = gtk_target_list_new (url_drag_types,
+						  G_N_ELEMENTS (url_drag_types));
+	priv->drag_actions = GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK;
+
 	gtk_entry_set_icon_drag_source (GTK_ENTRY (entry),
 					GTK_ENTRY_ICON_PRIMARY,
-					targetlist,
-					GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
-	gtk_target_list_unref (targetlist);
+					priv->drag_targets,
+					priv->drag_actions);
 
 	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (entry),
 					 GTK_ENTRY_ICON_PRIMARY,
 					 _("Drag and drop this icon to create a link to this page"));
 
-	gtk_entry_set_icon_activatable (GTK_ENTRY (entry),
-					GTK_ENTRY_ICON_PRIMARY,
-					TRUE);
-	gtk_entry_set_icon_activatable (GTK_ENTRY (entry),
-					GTK_ENTRY_ICON_SECONDARY,
-					TRUE);
+	gtk_drag_dest_set (entry,
+			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			   url_drag_types,
+			   G_N_ELEMENTS (url_drag_types),
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
 	g_object_connect (entry,
 			  "signal::icon-press", G_CALLBACK (icon_button_press_event_cb), lentry,
@@ -902,7 +923,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *lentry)
 			  "signal::drag-drop", G_CALLBACK (entry_drag_drop_cb), lentry,
 			  "signal::drag-data-get", G_CALLBACK (favicon_drag_data_get_cb), lentry,
 			  NULL);
-			  
+
 	g_signal_connect_after (entry, "key-press-event",
 				G_CALLBACK (entry_key_press_after_cb), lentry);
 	g_signal_connect_after (entry, "activate",
@@ -924,7 +945,6 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->user_changed = FALSE;
 	p->block_update = FALSE;
 	p->saved_text = NULL;
-	p->show_lock = FALSE;
 	p->show_favicon = TRUE;
 	p->dns_prefetch_handler = 0;
 
@@ -953,17 +973,10 @@ free_prefetch_helper (PrefetchHelper *helper)
 static gboolean
 do_dns_prefetch (PrefetchHelper *helper)
 {
-#ifdef HAVE_WEBKIT2
 	WebKitWebContext *context = webkit_web_context_get_default ();
 
 	if (helper->uri)
 		webkit_web_context_prefetch_dns (context, helper->uri->host);
-#else
-	SoupSession *session = webkit_get_default_session ();
-
-	if (helper->uri)
-		soup_session_prefetch_dns (session, helper->uri->host, NULL, NULL, NULL);
-#endif
 
 	helper->entry->priv->dns_prefetch_handler = 0;
 
@@ -993,6 +1006,7 @@ schedule_dns_prefetch (EphyLocationEntry *entry, guint interval, const gchar *ur
 		g_timeout_add_full (G_PRIORITY_DEFAULT, interval,
 				    (GSourceFunc) do_dns_prefetch, helper,
 				    (GDestroyNotify) free_prefetch_helper);
+	g_source_set_name_by_id (entry->priv->dns_prefetch_handler, "[epiphany] do_dns_prefetch");
 }
 
 static gboolean
@@ -1024,6 +1038,18 @@ cursor_on_match_cb  (GtkEntryCompletion *completion,
 	return TRUE;
 }
 
+static char *
+rgba_to_hex (GdkRGBA *color)
+{
+	char *hex;
+
+	hex = g_strdup_printf ("#%04X%04X%04X",
+			       (guint)(color->red * (gdouble)65535),
+			       (guint)(color->green * (gdouble)65535),
+			       (guint)(color->blue * (gdouble)65535));
+	return hex;
+}
+
 static void
 textcell_data_func (GtkCellLayout *cell_layout,
 		    GtkCellRenderer *cell,
@@ -1033,16 +1059,10 @@ textcell_data_func (GtkCellLayout *cell_layout,
 {
 	GtkWidget *entry;
 	EphyLocationEntryPrivate *priv;
-	PangoAttrList *list;
-	PangoAttribute *att;
-
 	char *ctext;
 	char *title;
 	char *url;
-
 	GtkStyleContext *style;
-	GdkRGBA color;
-
 	GValue text = { 0, };
 
 	entry = GTK_WIDGET (data);
@@ -1052,41 +1072,32 @@ textcell_data_func (GtkCellLayout *cell_layout,
 			priv->url_col, &url,
 			-1);
 
-	list = pango_attr_list_new ();
-
 	if (url)
 	{
-		/* Do not show internal ephy-about: protocol to users */
-		if (g_str_has_prefix (url, EPHY_ABOUT_SCHEME)) {
-			g_free (url);
-			url = g_strdup_printf ("about:%s", url + strlen (EPHY_ABOUT_SCHEME) + 1);
-		}
+		GdkRGBA color;
+		char *color_text;
 
-		ctext = g_strdup_printf ("%s\n%s", title, url);
+		sanitize_location (&url);
 
 		style = gtk_widget_get_style_context (entry);
 		gtk_style_context_get_color (style, GTK_STATE_FLAG_INSENSITIVE,
 					     &color);
 
-		att = pango_attr_foreground_new
-		   (color.red * 65535, color.green * 65535, color.blue * 65535);
-		att->start_index = strlen (title)+1;
-		pango_attr_list_insert (list, att);
+		color_text = rgba_to_hex (&color);
+		ctext = g_markup_printf_escaped ("%s\n<span font-size=\"small\" color=\"%s\">%s</span>", title, color_text, url);
+		g_free (color_text);
 	        g_free (title);
 	}
 	else
 	{
 		ctext = title;
 	}
-	g_object_set (cell, "attributes", list, NULL);
 
 	g_value_init (&text, G_TYPE_STRING);
 	g_value_take_string (&text, ctext);
-	g_object_set_property (G_OBJECT (cell), "text", &text);
+	g_object_set_property (G_OBJECT (cell), "markup", &text);
 	g_value_unset (&text);
 
-	pango_attr_list_unref (list);
-
 	g_free (url);
 }
 
@@ -1220,11 +1231,11 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	 * with anything in the entry.
 	 */
 
-#define ROW_PADDING_VERT 2
+#define ROW_PADDING_VERT 4
 
 #define ICON_PADDING_LEFT 5
 #define ICON_CONTENT_WIDTH 16
-#define ICON_PADDING_RIGHT 2
+#define ICON_PADDING_RIGHT 4
 
 #define ICON_CONTENT_HEIGHT 16
 
@@ -1338,10 +1349,18 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 			effective_text = g_strdup_printf ("about:%s",
 							  address + strlen (EPHY_ABOUT_SCHEME) + 1);
 		text = address;
+		gtk_entry_set_icon_drag_source (GTK_ENTRY (entry),
+						GTK_ENTRY_ICON_PRIMARY,
+						priv->drag_targets,
+						priv->drag_actions);
 	}
 	else
 	{
 		text = "";
+		gtk_entry_set_icon_drag_source (GTK_ENTRY (entry),
+						GTK_ENTRY_ICON_PRIMARY,
+						NULL,
+						GDK_ACTION_DEFAULT);
 	}
 
 	/* First record the new hash, then update the entry text */
@@ -1548,72 +1567,26 @@ ephy_location_entry_set_show_favicon (EphyLocationEntry *entry,
 }
 
 /**
- * ephy_location_entry_set_show_lock:
+ * ephy_location_entry_set_security_level:
  * @entry: an #EphyLocationEntry widget
- * @show_lock: if @entry should show a lock icon indicating the security level
- * of the page
+ * @state: the #EphySecurityLevel
  *
- * If @show_lock is TRUE, the location bar will show an icon reflecting the
- * security level of the page, by default it's shown only in secure and
- * insecure pages (insecure meaning secure pages with something broken in such
- * security)
+ * Set the lock icon to be displayed
  *
  **/
 void
-ephy_location_entry_set_show_lock (EphyLocationEntry *entry,
-				   gboolean show_lock)
+ephy_location_entry_set_security_level (EphyLocationEntry *entry,
+				        EphySecurityLevel security_level)
+
 {
-	EphyLocationEntryPrivate *priv;
+	const char *icon_name;
 
 	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
 
-	priv = entry->priv;
-
-	priv->show_lock = show_lock != FALSE;
-
-	gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry),
-				       GTK_ENTRY_ICON_SECONDARY,
-				       show_lock ? priv->lock_gicon : NULL);
-}
-
-/**
- * ephy_location_entry_set_lock_stock:
- * @entry: an #EphyLocationEntry widget
- * @stock_id: a stock_id from GTK+ stock icons
- *
- * Set the lock icon to be displayed, to actually show the icon see 
- * ephy_location_entry_set_show_lock.
- *
- **/
-void
-ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
-				    const char *stock_id)
-
-{
-	EphyLocationEntryPrivate *priv;
-
-	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
-
-	priv = entry->priv;
-
-	if (priv->lock_gicon)
-		g_object_unref (priv->lock_gicon);
-
-	/* At the moment we basically only show two kinds of
-	 * locks. Full/green for secure sites, Broken/red for sites
-	 * that are supposed to be secure but have some issues in
-	 * their security infrastructure (broken cert, etc). For
-	 * everything else, nothing is shown.
-	 */
-	if (!stock_id || g_str_equal (stock_id, STOCK_LOCK_BROKEN))
-		priv->lock_gicon = g_themed_icon_new_with_default_fallbacks ("channel-insecure-symbolic");
-	else
-		priv->lock_gicon = g_themed_icon_new_with_default_fallbacks ("channel-secure-symbolic");
-
-	if (priv->show_lock)
-		gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry),
-					       GTK_ENTRY_ICON_SECONDARY,
-					       priv->lock_gicon);
+	icon_name = ephy_security_level_to_icon_name (security_level);
+	gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
+					   GTK_ENTRY_ICON_SECONDARY,
+					   icon_name);
 }
 
 /**
