@@ -26,6 +26,7 @@
 #include "ephy-bookmarks-editor.h"
 #include "ephy-bookmarks-import.h"
 #include "ephy-debug.h"
+#include "ephy-download.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
@@ -35,17 +36,16 @@
 #include "ephy-lockdown.h"
 #include "ephy-prefs.h"
 #include "ephy-private.h"
-#include "ephy-search-provider.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-type-builtins.h"
 #include "ephy-web-view.h"
 #include "ephy-window.h"
-#include "pdm-dialog.h"
 #include "prefs-dialog.h"
 #include "window-commands.h"
 
 #include <glib/gi18n.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
@@ -54,7 +54,6 @@
 #define EPHY_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_SHELL, EphyShellPrivate))
 
 struct _EphyShellPrivate {
-  EphySearchProvider *search_provider;
   EphySession *session;
   GList *windows;
   GObject *lockdown;
@@ -62,11 +61,10 @@ struct _EphyShellPrivate {
   GNetworkMonitor *network_monitor;
   GtkWidget *bme;
   GtkWidget *history_window;
-  GObject *pdm_dialog;
   GObject *prefs_dialog;
-  GList *del_on_exit;
-  EphyShellStartupContext *startup_context;
-  guint open_uris_idle_id;
+  EphyShellStartupContext *local_startup_context;
+  EphyShellStartupContext *remote_startup_context;
+  GSList *open_uris_idle_ids;
 };
 
 EphyShell *ephy_shell = NULL;
@@ -115,16 +113,27 @@ ephy_shell_startup_context_new (EphyStartupFlags startup_flags,
 }
 
 static void
-ephy_shell_startup_continue (EphyShell *shell)
+ephy_shell_startup_context_free (EphyShellStartupContext *ctx)
+{
+  g_assert (ctx != NULL);
+
+  g_free (ctx->bookmarks_filename);
+  g_free (ctx->session_filename);
+  g_free (ctx->bookmark_url);
+
+  g_strfreev (ctx->arguments);
+
+  g_slice_free (EphyShellStartupContext, ctx);
+}
+
+static void
+ephy_shell_startup_continue (EphyShell *shell, EphyShellStartupContext *ctx)
 {
-  EphyShellStartupContext *ctx;
   EphySession *session;
 
   session = ephy_shell_get_session (shell);
   g_assert (session != NULL);
 
-  ctx = shell->priv->startup_context;
-
   if (ctx->session_filename != NULL)
     ephy_session_load (session, (const char *)ctx->session_filename,
                        ctx->user_time, NULL, NULL, NULL);
@@ -139,7 +148,8 @@ ephy_shell_startup_continue (EphyShell *shell)
    * otherwise our first call to ephy_window_is_on_current_workspace
    * will be unreliable.
    */
-  wnck_screen_force_update (wnck_screen_get_default ());
+  if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    wnck_screen_force_update (wnck_screen_get_default ());
 }
 
 static void
@@ -179,7 +189,11 @@ show_history (GSimpleAction *action,
               GVariant *parameter,
               gpointer user_data)
 {
-  window_cmd_edit_history (NULL, NULL);
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_edit_history (NULL, EPHY_WINDOW (window));
 }
 
 static void
@@ -187,15 +201,11 @@ show_preferences (GSimpleAction *action,
                   GVariant *parameter,
                   gpointer user_data)
 {
-  window_cmd_edit_preferences (NULL, NULL);
-}
+  GtkWindow *window;
 
-static void
-show_pdm (GSimpleAction *action,
-          GVariant *parameter,
-          gpointer user_data)
-{
-  window_cmd_edit_personal_data (NULL, NULL);
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_edit_preferences (NULL, EPHY_WINDOW (window));
 }
 
 static void
@@ -231,12 +241,11 @@ quit_application (GSimpleAction *action,
 }
 
 static GActionEntry app_entries[] = {
-  { "new", new_window, NULL, NULL, NULL },
+  { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
   { "bookmarks", show_bookmarks, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
-  { "pdm", show_pdm, NULL, NULL, NULL },
   { "help", show_help, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
@@ -291,49 +300,35 @@ session_load_cb (GObject *object,
                  gpointer user_data)
 {
   EphySession *session = EPHY_SESSION (object);
-  EphyShell *shell = EPHY_SHELL (user_data);
+  EphyShellStartupContext *ctx = (EphyShellStartupContext *)user_data;
 
-  if (ephy_session_resume_finish (session, result, NULL))
-    shell->priv->startup_context->startup_flags |= EPHY_STARTUP_RESUMING_SESSION;
-
-  ephy_shell_startup_continue (shell);
-}
-
-static void
-ephy_shell_start_headless (EphyShell *shell)
-{
-  /* A bit of a hack: because we don't pass IS_SERVICE to GApplication
-     (because otherwise we would need a separate launcher binary), we
-     don't get the 10 seconds timeout for the first DBus call.
-     So just hold/release to get the inactivity timeout started instead.
-  */
-
-  g_application_hold (G_APPLICATION (shell));
-  g_application_release (G_APPLICATION (shell));
-  ephy_shell_startup_continue (shell);
+  ephy_session_resume_finish (session, result, NULL);
+  ephy_shell_startup_continue (ephy_shell_get_default (), ctx);
 }
 
 static void
 ephy_shell_activate (GApplication *application)
 {
   EphyShell *shell = EPHY_SHELL (application);
+  EphyShellPrivate *priv = shell->priv;
 
   /*
    * We get here on each new instance (remote or not). Autoresume the
-   * session unless we are in application or headless mode and queue the
-   * commands.
+   * session and queue the commands if we are a local instance. Otherwise,
+   * execute the commands immediately, before the remote startup context
+   * can be invalidated by another remote instance.
    */
-  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    EphyShellStartupContext *ctx;
-
-    ctx = shell->priv->startup_context;
-    if (ctx->startup_flags != EPHY_STARTUP_OPEN_NOTHING) {
+  if (priv->remote_startup_context == NULL) {
+    if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
       ephy_session_resume (ephy_shell_get_session (shell),
-                           ctx->user_time, NULL, session_load_cb, shell);
+                           priv->local_startup_context->user_time,
+                           NULL, session_load_cb, priv->local_startup_context);
     } else
-      ephy_shell_start_headless (shell);
-  } else
-    ephy_shell_startup_continue (shell);
+      ephy_shell_startup_continue (shell, priv->local_startup_context);
+  } else {
+    ephy_shell_startup_continue (shell, priv->remote_startup_context);
+    g_clear_pointer (&priv->remote_startup_context, ephy_shell_startup_context_free);
+  }
 }
 
 /*
@@ -365,13 +360,13 @@ ephy_shell_add_platform_data (GApplication *application,
   G_APPLICATION_CLASS (ephy_shell_parent_class)->add_platform_data (application,
                                                                     builder);
 
-  if (app->priv->startup_context) {
+  if (app->priv->local_startup_context) {
     /*
      * We create an array variant that contains only the elements in
      * ctx that are non-NULL.
      */
     ctx_builder = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);
-    ctx = app->priv->startup_context;
+    ctx = app->priv->local_startup_context;
 
     if (ctx->startup_flags)
       g_variant_builder_add (ctx_builder, "{iv}",
@@ -418,24 +413,6 @@ ephy_shell_add_platform_data (GApplication *application,
   }
 }
 
-static void
-ephy_shell_free_startup_context (EphyShell *shell)
-{
-  EphyShellStartupContext *ctx = shell->priv->startup_context;
-
-  g_assert (ctx != NULL);
-
-  g_free (ctx->bookmarks_filename);
-  g_free (ctx->session_filename);
-  g_free (ctx->bookmark_url);
-
-  g_strfreev (ctx->arguments);
-
-  g_slice_free (EphyShellStartupContext, ctx);
-
-  shell->priv->startup_context = NULL;
-}
-
 static void
 ephy_shell_before_emit (GApplication *application,
                         GVariant *platform_data)
@@ -486,9 +463,9 @@ ephy_shell_before_emit (GApplication *application,
     }
   }
 
-  if (shell->priv->startup_context)
-    ephy_shell_free_startup_context (shell);
-  shell->priv->startup_context = ctx;
+  /* We have already processed and discarded any previous remote startup contexts. */
+  g_assert (shell->priv->remote_startup_context == NULL);
+  shell->priv->remote_startup_context = ctx;
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->before_emit (application,
                                                               platform_data);
@@ -523,42 +500,6 @@ ephy_shell_constructed (GObject *object)
     G_OBJECT_CLASS (ephy_shell_parent_class)->constructed (object);
 }
 
-static gboolean
-ephy_shell_dbus_register (GApplication    *application,
-                          GDBusConnection *connection,
-                          const gchar     *object_path,
-                          GError         **error)
-{
-  EphyShell *self;
-
-  if (!G_APPLICATION_CLASS (ephy_shell_parent_class)->dbus_register (application,
-                                                                     connection,
-                                                                     object_path,
-                                                                     error))
-    return FALSE;
-
-  self = EPHY_SHELL (application);
-
-  return ephy_search_provider_dbus_register (self->priv->search_provider, connection,
-                                             object_path, error);
-}
-
-static void
-ephy_shell_dbus_unregister (GApplication    *application,
-                            GDBusConnection *connection,
-                            const gchar     *object_path)
-{
-  EphyShell *self;
-
-  self = EPHY_SHELL (application);
-  if (self->priv->search_provider)
-    ephy_search_provider_dbus_unregister (self->priv->search_provider, connection, object_path);
-
-  G_APPLICATION_CLASS (ephy_shell_parent_class)->dbus_unregister (application,
-                                                                  connection,
-                                                                  object_path);
-}
-
 static void
 ephy_shell_class_init (EphyShellClass *klass)
 {
@@ -573,13 +514,10 @@ ephy_shell_class_init (EphyShellClass *klass)
   application_class->activate = ephy_shell_activate;
   application_class->before_emit = ephy_shell_before_emit;
   application_class->add_platform_data = ephy_shell_add_platform_data;
-  application_class->dbus_register = ephy_shell_dbus_register;
-  application_class->dbus_unregister = ephy_shell_dbus_unregister;
 
   g_type_class_add_private (object_class, sizeof(EphyShellPrivate));
 }
 
-#ifdef HAVE_WEBKIT2
 static void
 download_started_cb (WebKitWebContext *web_context,
                      WebKitDownload *download,
@@ -587,6 +525,7 @@ download_started_cb (WebKitWebContext *web_context,
 {
   GtkWindow *window = NULL;
   WebKitWebView *web_view;
+  EphyDownload *ephy_download;
   gboolean ephy_download_set;
 
   /* Is download locked down? */
@@ -617,20 +556,18 @@ download_started_cb (WebKitWebContext *web_context,
   if (!window)
     window = gtk_application_get_active_window (GTK_APPLICATION (shell));
 
-  ephy_download_new_for_download (download, window);
+  ephy_download = ephy_download_new (download, window);
+  ephy_window_add_download (EPHY_WINDOW (window), ephy_download);
+  g_object_unref (ephy_download);
 }
-#endif
 
 static void
 ephy_shell_init (EphyShell *shell)
 {
   EphyShell **ptr = &ephy_shell;
-
-#ifdef HAVE_WEBKIT2
   WebKitWebContext *web_context;
   EphyEmbedShellMode mode;
   char *favicon_db_path;
-#endif
 
   shell->priv = EPHY_SHELL_GET_PRIVATE (shell);
 
@@ -640,12 +577,14 @@ ephy_shell_init (EphyShell *shell)
   g_object_add_weak_pointer (G_OBJECT (ephy_shell),
                              (gpointer *)ptr);
 
-#ifdef HAVE_WEBKIT2
   web_context = webkit_web_context_get_default ();
   g_signal_connect (web_context, "download-started",
                     G_CALLBACK (download_started_cb),
                     shell);
 
+  /* Do not ignore TLS errors. */
+  webkit_web_context_set_tls_errors_policy (web_context, WEBKIT_TLS_ERRORS_POLICY_FAIL);
+
   /* Initialize the favicon cache as early as possible, or further
      calls to webkit_web_context_get_favicon_database will fail. */
   mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
@@ -655,11 +594,12 @@ ephy_shell_init (EphyShell *shell)
 
   webkit_web_context_set_favicon_database_directory (web_context, favicon_db_path);
   g_free (favicon_db_path);
-#endif
+}
 
-  shell->priv->search_provider = ephy_search_provider_new ();
-
-  g_application_set_inactivity_timeout (G_APPLICATION (shell), 60 * 1000);
+static void
+remove_open_uris_idle_cb (gpointer data)
+{
+  g_source_remove (GPOINTER_TO_UINT (data));
 }
 
 static void
@@ -669,20 +609,16 @@ ephy_shell_dispose (GObject *object)
 
   LOG ("EphyShell disposing");
 
-  g_clear_object (&priv->search_provider);
   g_clear_object (&priv->session);
   g_clear_object (&priv->lockdown);
   g_clear_pointer (&priv->bme, gtk_widget_destroy);
   g_clear_pointer (&priv->history_window, gtk_widget_destroy);
-  g_clear_object (&priv->pdm_dialog);
   g_clear_object (&priv->prefs_dialog);
   g_clear_object (&priv->bookmarks);
   g_clear_object (&priv->network_monitor);
 
-  if (priv->open_uris_idle_id > 0)  {
-    g_source_remove (priv->open_uris_idle_id);
-    priv->open_uris_idle_id = 0;
-  }
+  g_slist_free_full (priv->open_uris_idle_ids, remove_open_uris_idle_cb);
+  priv->open_uris_idle_ids = NULL;
 
   G_OBJECT_CLASS (ephy_shell_parent_class)->dispose (object);
 }
@@ -691,9 +627,10 @@ static void
 ephy_shell_finalize (GObject *object)
 {
   EphyShell *shell = EPHY_SHELL (object);
+  EphyShellPrivate *priv = shell->priv;
 
-  if (shell->priv->startup_context)
-    ephy_shell_free_startup_context (shell);
+  g_clear_pointer (&priv->local_startup_context, ephy_shell_startup_context_free);
+  g_clear_pointer (&priv->remote_startup_context, ephy_shell_startup_context_free);
 
   G_OBJECT_CLASS (ephy_shell_parent_class)->finalize (object);
 
@@ -716,11 +653,8 @@ ephy_shell_get_default (void)
 /**
  * ephy_shell_new_tab_full:
  * @shell: a #EphyShell
- * @parent_window: the target #EphyWindow or %NULL
+ * @window: the target #EphyWindow or %NULL
  * @previous_embed: the referrer embed, or %NULL
- * @request: a #WebKitNetworkRequest to load or %NULL
- * @chrome: a #EphyEmbedChrome mask to use if creating a new window
- * @is_popup: whether the new window is a popup
  * @user_time: a timestamp, or 0
  *
  * Create a new tab and the parent window when necessary.
@@ -730,61 +664,29 @@ ephy_shell_get_default (void)
  **/
 EphyEmbed *
 ephy_shell_new_tab_full (EphyShell *shell,
-                         EphyWindow *parent_window,
+                         const char *title,
+                         WebKitWebView *related_view,
+                         EphyWindow *window,
                          EphyEmbed *previous_embed,
-#ifdef HAVE_WEBKIT2
-                         WebKitURIRequest *request,
-#else
-                         WebKitNetworkRequest *request,
-#endif
                          EphyNewTabFlags flags,
-                         EphyWebViewChrome chrome,
-                         gboolean is_popup,
                          guint32 user_time)
 {
   EphyEmbedShell *embed_shell;
-  EphyWindow *window;
+  GtkWidget *web_view;
   EphyEmbed *embed = NULL;
-  gboolean fullscreen_lockdown = FALSE;
-  gboolean in_new_window = TRUE;
-  gboolean open_page = FALSE;
-  gboolean delayed_open_page = FALSE;
   gboolean jump_to = FALSE;
-  gboolean active_is_blank = FALSE;
-  gboolean copy_history = TRUE;
-  gboolean is_empty = FALSE;
   int position = -1;
 
   g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
-  g_return_val_if_fail (EPHY_IS_WINDOW (parent_window) || !parent_window, NULL);
+  g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
   g_return_val_if_fail (EPHY_IS_EMBED (previous_embed) || !previous_embed, NULL);
-#ifdef HAVE_WEBKIT2
-  g_return_val_if_fail (WEBKIT_IS_URI_REQUEST (request) || !request, NULL);
-#else
-  g_return_val_if_fail (WEBKIT_IS_NETWORK_REQUEST (request) || !request, NULL);
-#endif
 
   embed_shell = EPHY_EMBED_SHELL (shell);
 
-  if (flags & EPHY_NEW_TAB_OPEN_PAGE) open_page = TRUE;
-  if (flags & EPHY_NEW_TAB_DELAYED_OPEN_PAGE) delayed_open_page = TRUE;
-  if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
-  if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
-  if (flags & EPHY_NEW_TAB_DONT_COPY_HISTORY) copy_history = FALSE;
   if (flags & EPHY_NEW_TAB_JUMP) jump_to = TRUE;
 
-  fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                                                EPHY_PREFS_LOCKDOWN_FULLSCREEN);
-  in_new_window = in_new_window && !fullscreen_lockdown;
-  g_return_val_if_fail ((open_page || delayed_open_page) == (gboolean)(request != NULL), NULL);
-
-  LOG ("Opening new tab parent-window %p parent-embed %p in-new-window:%s jump-to:%s",
-       parent_window, previous_embed, in_new_window ? "t" : "f", jump_to ? "t" : "f");
-
-  if (!in_new_window && parent_window != NULL)
-    window = parent_window;
-  else
-    window = ephy_window_new_with_chrome (chrome, is_popup);
+  LOG ("Opening new tab window %p parent-embed %p jump-to:%s",
+       window, previous_embed, jump_to ? "t" : "f");
 
   if (flags & EPHY_NEW_TAB_APPEND_AFTER) {
     if (previous_embed) {
@@ -799,30 +701,17 @@ ephy_shell_new_tab_full (EphyShell *shell,
   if (flags & EPHY_NEW_TAB_FIRST)
     position = 0;
 
-  if (flags & EPHY_NEW_TAB_FROM_EXTERNAL) {
-    /* If the active embed is blank, use that to open the url and jump to it */
-    embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    if (embed != NULL) {
-      EphyWebView *view = ephy_embed_get_web_view (embed);
-      if ((ephy_web_view_get_is_blank (view) || ephy_embed_get_overview_mode (embed)) &&
-          ephy_web_view_is_loading (view) == FALSE) {
-        active_is_blank = TRUE;
-      }
-    }
-  }
+  if (related_view)
+    web_view = ephy_web_view_new_with_related_view (related_view);
+  else
+    web_view = ephy_web_view_new ();
 
-  if (active_is_blank == FALSE) {
-    embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED, NULL));
-    g_assert (embed != NULL);
-    gtk_widget_show (GTK_WIDGET (embed));
-
-    ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
-  }
-
-  if (copy_history && previous_embed != NULL) {
-    ephy_web_view_copy_back_history (ephy_embed_get_web_view (previous_embed),
-                                     ephy_embed_get_web_view (embed));
-  }
+  embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED,
+                                    "web-view", web_view,
+                                    "title", title,
+                                    NULL));
+  gtk_widget_show (GTK_WIDGET (embed));
+  ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
 
   ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
 
@@ -831,50 +720,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
     gtk_widget_show (GTK_WIDGET (window));
   }
 
-  if (flags & EPHY_NEW_TAB_FULLSCREEN_MODE) {
-    gtk_window_fullscreen (GTK_WINDOW (window));
-  }
-
-  if (flags & EPHY_NEW_TAB_HOME_PAGE ||
-      flags & EPHY_NEW_TAB_NEW_PAGE) {
-    EphyWebView *view = ephy_embed_get_web_view (embed);
-    ephy_web_view_set_typed_address (view, "");
-    ephy_window_activate_location (window);
-    ephy_web_view_load_homepage (view);
-    is_empty = TRUE;
-  } else if (open_page) {
-    ephy_web_view_load_request (ephy_embed_get_web_view (embed),
-                                request);
-
-#ifdef HAVE_WEBKIT2
-    is_empty = ephy_embed_utils_url_is_empty (webkit_uri_request_get_uri (request));
-#else
-    is_empty = ephy_embed_utils_url_is_empty (webkit_network_request_get_uri (request));
-#endif
-  } else if (delayed_open_page)
-      ephy_embed_set_delayed_load_request (embed, request);
-
-  /* Make sure the initial focus is somewhere sensible and not, for
-   * example, on the reload button.
-   */
-  if (in_new_window || jump_to) {
-    /* If the location entry is blank, focus that, except if the
-     * page was a copy */
-    if (is_empty) {
-      /* empty page, focus location entry */
-      ephy_window_activate_location (window);
-    } else if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 && embed != NULL &&
-               ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST) {
-      /* non-empty page, focus the page. but make sure the widget is realised first! */
-      gtk_widget_realize (GTK_WIDGET (embed));
-      gtk_widget_grab_focus (GTK_WIDGET (embed));
-    }
-  }
-
-  if (flags & EPHY_NEW_TAB_PRESENT_WINDOW &&
-      ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (embed_shell)) != EPHY_EMBED_SHELL_MODE_TEST)
-    gtk_window_present_with_time (GTK_WINDOW (window), user_time);
-
   return embed;
 }
 
@@ -883,7 +728,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
  * @shell: a #EphyShell
  * @parent_window: the target #EphyWindow or %NULL
  * @previous_embed: the referrer embed, or %NULL
- * @url: an url to load or %NULL
  *
  * Create a new tab and the parent window when necessary.
  * Use this function to open urls in new window/tabs.
@@ -894,24 +738,11 @@ EphyEmbed *
 ephy_shell_new_tab (EphyShell *shell,
                     EphyWindow *parent_window,
                     EphyEmbed *previous_embed,
-                    const char *url,
                     EphyNewTabFlags flags)
 {
-  EphyEmbed *embed;
-#ifdef HAVE_WEBKIT2
-  WebKitURIRequest *request = url ? webkit_uri_request_new (url) : NULL;
-#else
-  WebKitNetworkRequest *request = url ? webkit_network_request_new (url) : NULL;
-#endif
-
-  embed = ephy_shell_new_tab_full (shell, parent_window,
-                                   previous_embed, request, flags,
-                                   EPHY_WEB_VIEW_CHROME_ALL, FALSE, 0);
-
-  if (request)
-    g_object_unref (request);
-
-  return embed;
+  return ephy_shell_new_tab_full (shell, NULL, NULL, parent_window,
+                                  previous_embed, flags,
+                                  0);
 }
 
 /**
@@ -1000,33 +831,15 @@ ephy_shell_get_history_window (EphyShell *shell)
     service = EPHY_HISTORY_SERVICE
       (ephy_embed_shell_get_global_history_service (embed_shell));
     shell->priv->history_window = ephy_history_window_new (service);
+    g_signal_connect (shell->priv->history_window,
+                      "destroy",
+                      G_CALLBACK (gtk_widget_destroyed),
+                      &shell->priv->history_window);
   }
 
   return shell->priv->history_window;
 }
 
-/**
- * ephy_shell_get_pdm_dialog:
- *
- * Return value: (transfer none):
- **/
-GObject *
-ephy_shell_get_pdm_dialog (EphyShell *shell)
-{
-  if (shell->priv->pdm_dialog == NULL) {
-    GObject **dialog;
-
-    shell->priv->pdm_dialog = g_object_new (EPHY_TYPE_PDM_DIALOG, NULL);
-
-    dialog = &shell->priv->pdm_dialog;
-
-    g_object_add_weak_pointer (shell->priv->pdm_dialog,
-                               (gpointer *)dialog);
-  }
-
-  return shell->priv->pdm_dialog;
-}
-
 /**
  * ephy_shell_get_prefs_dialog:
  *
@@ -1036,14 +849,13 @@ GObject *
 ephy_shell_get_prefs_dialog (EphyShell *shell)
 {
   if (shell->priv->prefs_dialog == NULL) {
-    GObject **dialog;
-
-    shell->priv->prefs_dialog = g_object_new (EPHY_TYPE_PREFS_DIALOG, NULL);
-
-    dialog  = &shell->priv->prefs_dialog;
-
-    g_object_add_weak_pointer (shell->priv->prefs_dialog,
-                               (gpointer *)dialog);
+    shell->priv->prefs_dialog = g_object_new (EPHY_TYPE_PREFS_DIALOG,
+                                              "use-header-bar", TRUE,
+                                              NULL);
+    g_signal_connect (shell->priv->prefs_dialog,
+                      "destroy",
+                      G_CALLBACK (gtk_widget_destroyed),
+                      &shell->priv->prefs_dialog);
   }
 
   return shell->priv->prefs_dialog;
@@ -1067,7 +879,7 @@ _ephy_shell_create_instance (EphyEmbedShellMode mode)
  * @shell: A #EphyShell
  * @ctx: (transfer full): a #EphyShellStartupContext
  *
- * Sets the startup context to be used during activation of a new instance.
+ * Sets the local startup context to be used during activation of a new instance.
  * See ephy_shell_set_startup_new().
  **/
 void
@@ -1076,10 +888,9 @@ ephy_shell_set_startup_context (EphyShell *shell,
 {
   g_return_if_fail (EPHY_IS_SHELL (shell));
 
-  if (shell->priv->startup_context)
-    ephy_shell_free_startup_context (shell);
+  g_assert (shell->priv->local_startup_context == NULL);
 
-  shell->priv->startup_context = ctx;
+  shell->priv->local_startup_context = ctx;
 }
 
 guint
@@ -1154,6 +965,15 @@ ephy_shell_close_all_windows (EphyShell *shell)
   return retval;
 }
 
+static gboolean
+tab_is_empty (EphyEmbed *embed)
+{
+  EphyWebView *view = ephy_embed_get_web_view (embed);
+
+  return ((ephy_web_view_get_is_blank (view) || ephy_web_view_is_overview (view)) &&
+          !ephy_web_view_is_loading (view));
+}
+
 typedef struct {
   EphyShell *shell;
   EphySession *session;
@@ -1161,7 +981,10 @@ typedef struct {
   char **uris;
   EphyNewTabFlags flags;
   guint32 user_time;
+  EphyEmbed *previous_embed;
   guint current_uri;
+  gboolean reuse_empty_tab;
+  guint source_id;
 } OpenURIsData;
 
 static OpenURIsData *
@@ -1172,6 +995,7 @@ open_uris_data_new (EphyShell *shell,
 {
   OpenURIsData *data;
   gboolean new_windows_in_tabs;
+  gboolean fullscreen_lockdown;
   gboolean have_uris;
 
   data = g_slice_new0 (OpenURIsData);
@@ -1180,24 +1004,21 @@ open_uris_data_new (EphyShell *shell,
   data->uris = g_strdupv ((char **)uris);
   data->user_time = user_time;
 
-  data->window = ephy_shell_get_main_window (shell);
-
   new_windows_in_tabs = g_settings_get_boolean (EPHY_SETTINGS_MAIN,
                                                 EPHY_PREFS_NEW_WINDOWS_IN_TABS);
+  fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                                                EPHY_PREFS_LOCKDOWN_FULLSCREEN);
 
   have_uris = ! (g_strv_length ((char **)uris) == 1 && g_str_equal (uris[0], ""));
 
-  if (startup_flags & EPHY_STARTUP_NEW_TAB)
-    data->flags |= EPHY_NEW_TAB_FROM_EXTERNAL;
-
-  if (startup_flags & EPHY_STARTUP_NEW_WINDOW) {
-    data->window = NULL;
-    data->flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
+  if (startup_flags & EPHY_STARTUP_NEW_WINDOW && !fullscreen_lockdown) {
+    data->window = ephy_window_new ();
   } else if (startup_flags & EPHY_STARTUP_NEW_TAB || (new_windows_in_tabs && have_uris)) {
-    data->flags |= EPHY_NEW_TAB_IN_EXISTING_WINDOW | EPHY_NEW_TAB_JUMP | EPHY_NEW_TAB_PRESENT_WINDOW | EPHY_NEW_TAB_FROM_EXTERNAL;
-  } else if (!have_uris && !(startup_flags & EPHY_STARTUP_RESUMING_SESSION)) {
-    data->window = NULL;
-    data->flags |= EPHY_NEW_TAB_IN_NEW_WINDOW;
+    data->flags |= EPHY_NEW_TAB_JUMP;
+    data->window = ephy_shell_get_main_window (shell);
+    data->reuse_empty_tab = TRUE;
+  } else if (!have_uris) {
+    data->window = ephy_window_new ();
   }
 
   g_application_hold (G_APPLICATION (shell));
@@ -1218,44 +1039,46 @@ open_uris_data_free (OpenURIsData *data)
 static gboolean
 ephy_shell_open_uris_idle (OpenURIsData *data)
 {
-  EphyEmbed *embed;
-  EphyNewTabFlags page_flags;
+  EphyEmbed *embed = NULL;
+  EphyNewTabFlags page_flags = 0;
+  gboolean reusing_empty_tab = FALSE;
   const char *url;
-#ifdef HAVE_WEBKIT2
-  WebKitURIRequest *request = NULL;
-#else
-  WebKitNetworkRequest *request = NULL;
-#endif
 
-  if (!data->window && !data->flags)
-    return FALSE;
-
-  url = data->uris[data->current_uri];
-  if (url[0] == '\0') {
-    page_flags = EPHY_NEW_TAB_HOME_PAGE;
-  } else {
-    page_flags = EPHY_NEW_TAB_OPEN_PAGE;
-#ifdef HAVE_WEBKIT2
-    request = webkit_uri_request_new (url);
-#else
-    request = webkit_network_request_new (url);
-#endif
+  if (!data->window)
+    data->window = ephy_window_new ();
+  else if (data->previous_embed)
+    page_flags |= EPHY_NEW_TAB_APPEND_AFTER;
+  else if (data->reuse_empty_tab) {
+    embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (data->window));
+    reusing_empty_tab = tab_is_empty (embed);
   }
 
-  embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
-                                   data->window,
-                                   NULL /* parent tab */,
-                                   request,
-                                   data->flags | page_flags,
-                                   EPHY_WEB_VIEW_CHROME_ALL,
-                                   FALSE /* is popup? */,
-                                   data->user_time);
+  if (!reusing_empty_tab) {
+    embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
+                                     NULL, NULL,
+                                     data->window,
+                                     data->previous_embed,
+                                     data->flags | page_flags,
+                                     data->user_time);
+  }
 
-  if (request)
-    g_object_unref (request);
+  url = data->uris[data->current_uri];
+  if (url[0] != '\0') {
+    ephy_web_view_load_url (ephy_embed_get_web_view (embed), url);
+
+    /* When reusing an empty tab, the focus is in the location entry */
+    if (reusing_empty_tab || data->flags & EPHY_NEW_TAB_JUMP)
+      gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+    if (data->flags & EPHY_NEW_TAB_JUMP && ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (data->shell)) != EPHY_EMBED_SHELL_MODE_TEST)
+      gtk_window_present_with_time (GTK_WINDOW (data->window), data->user_time);
+  } else {
+    ephy_web_view_load_homepage (ephy_embed_get_web_view (embed));
+    ephy_window_activate_location (data->window);
+  }
 
-  data->window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
   data->current_uri++;
+  data->previous_embed = embed;
 
   return data->uris[data->current_uri] != NULL;
 }
@@ -1263,7 +1086,8 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
 static void
 ephy_shell_open_uris_idle_done (OpenURIsData *data)
 {
-  data->shell->priv->open_uris_idle_id = 0;
+  data->shell->priv->open_uris_idle_ids = g_slist_remove (data->shell->priv->open_uris_idle_ids,
+                                                          GUINT_TO_POINTER (data->source_id));
   open_uris_data_free (data);
 }
 
@@ -1273,14 +1097,18 @@ ephy_shell_open_uris (EphyShell *shell,
                       EphyStartupFlags startup_flags,
                       guint32 user_time)
 {
+  OpenURIsData *data;
+  guint id;
+
   g_return_if_fail (EPHY_IS_SHELL (shell));
 
-  if (shell->priv->open_uris_idle_id == 0) {
-    shell->priv->open_uris_idle_id =
-      g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-                       (GSourceFunc)ephy_shell_open_uris_idle,
-                       open_uris_data_new (shell, uris, startup_flags, user_time),
-                       (GDestroyNotify)ephy_shell_open_uris_idle_done);
-  }
+  data = open_uris_data_new (shell, uris, startup_flags, user_time);
+  id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                        (GSourceFunc)ephy_shell_open_uris_idle,
+                        data,
+                        (GDestroyNotify)ephy_shell_open_uris_idle_done);
+  data->source_id = id;
+
+  shell->priv->open_uris_idle_ids = g_slist_prepend (shell->priv->open_uris_idle_ids, GUINT_TO_POINTER (id));
 }
 
